package com.libreoffice;
import java.io.File;
import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.bridge.XUnoUrlResolver;
import com.sun.star.container.NoSuchElementException;
import com.sun.star.container.XNameAccess;
import com.sun.star.document.XDocumentInsertable;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XStorable;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.text.XBookmarksSupplier;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextCursor;
import com.sun.star.text.XTextDocument;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.util.XCloseable;


/** The purpose of this class is to read a teamplate  document and save it after editing this
 * file to a specified URL. 
 */
public class DocumentGenerator {
    /** The main method of the application.
     * @param args The program needs two arguments:
     * - full template file name to open,
     * - full file name to save.
     *   full html file name to insert content
     */
	
	protected static XComponentContext xRemoteContext;
	static DocumentGenerator docGenrator;
    public static void main(String args[]) throws Exception {
        if ( args.length < 3 &&  args[0].length()>0 && args[1].length()>0 && args[2].length()>0 ) {
            System.out.println("usage: java -jar document-generator.jar" +
                               "\"<URL|path to load>\" \"<URL|path to save>\" \"<URL|path to instert content>\"" );
            System.out.println("\ne.g.:");
            System.out.println("java -jar document-generator.jar " +
                               "\"file:///f:/template.doc\"" +
                               "\"file:///f:/output.doc\"" +
                               "\"file:///f:/source_html.html\"");
            System.exit(1);
        }
        java.io.File sourceFile = new java.io.File(args[0]);
        StringBuffer sLoadUrl = new StringBuffer("file:///");
        sLoadUrl.append(sourceFile.getCanonicalPath().replace('\\', '/'));

        sourceFile = new java.io.File(args[1]);
        StringBuffer sSaveUrl = new StringBuffer("file:///");
        sSaveUrl.append(sourceFile.getCanonicalPath().replace('\\', '/'));
        sourceFile = new java.io.File(args[2]);
        StringBuffer sHtmlUrl = new StringBuffer("file:///");
        sHtmlUrl.append(sourceFile.getCanonicalPath().replace('\\', '/'));
        StringBuffer temp=new StringBuffer( args[1].toString().substring(0,args[1].toString().lastIndexOf('\\')+1));
         StringBuffer []sargs={sLoadUrl,sSaveUrl,sHtmlUrl,temp};
         docGenrator=new DocumentGenerator();
         docGenrator.generateDoc(sargs);
    }
    
    

    protected static synchronized XMultiComponentFactory getRemoteServiceManager(String unoUrl)
			throws java.lang.Exception {
		if (xRemoteContext == null) {
			XComponentContext xLocalContext = com.sun.star.comp.helper.Bootstrap.createInitialComponentContext(null);
			XMultiComponentFactory xLocalServiceManager = xLocalContext.getServiceManager();
			Object urlResolver = xLocalServiceManager.createInstanceWithContext("com.sun.star.bridge.UnoUrlResolver",
					xLocalContext);
			XUnoUrlResolver xUnoUrlResolver = (XUnoUrlResolver) UnoRuntime.queryInterface(XUnoUrlResolver.class,
					urlResolver);
						Object initialObject = xUnoUrlResolver.resolve(unoUrl);
			XPropertySet xPropertySet = (XPropertySet) UnoRuntime.queryInterface(XPropertySet.class, initialObject);
			Object context = xPropertySet.getPropertyValue("DefaultContext");
			xRemoteContext = (XComponentContext) UnoRuntime.queryInterface(XComponentContext.class, context);
		}
		return xRemoteContext.getServiceManager();
	}
    
    public XTextDocument open(String docName, String sConnectionString, StringBuffer errorMsg) {
		XTextDocument doc = null;
		String strFormat = getFileExtension(docName);
		try {

			XMultiComponentFactory xmulticomponentfactory = null;
			try {
				xmulticomponentfactory = getRemoteServiceManager(sConnectionString);
			} catch (com.sun.star.lang.DisposedException e) {
				xRemoteContext = null;
				xmulticomponentfactory = getRemoteServiceManager(sConnectionString);
			}

			XComponentLoader xcomponentloader = (XComponentLoader) UnoRuntime.queryInterface(XComponentLoader.class,
					xmulticomponentfactory.createInstanceWithContext("com.sun.star.frame.Desktop", xRemoteContext));

			PropertyValue[] propertyvalue = new PropertyValue[2];
			propertyvalue[0] = new PropertyValue();
			propertyvalue[0].Name = "Hidden";
			propertyvalue[0].Value = new Boolean(true);
			propertyvalue[1] = new PropertyValue();
			propertyvalue[1].Name = "FilterName";
			propertyvalue[1].Value =getDocumentFormatFilterNameText(strFormat);

			XComponent docComponent = xcomponentloader.loadComponentFromURL(docName, "_blank", 0, propertyvalue);
			doc = (XTextDocument) UnoRuntime.queryInterface(XTextDocument.class, docComponent);

		} catch (Exception exception) {
			System.out.println("No connection found to Word Doc service provider ");
			errorMsg.append(exception.toString());
			doc = null;
		}
		return doc;
	}
    
	public void save(XTextDocument doc, String targetDocName, String newDocName, StringBuffer errorMsg) {
		try {
			String strFormat=getFileExtension(targetDocName);
			XStorable xstorable = (XStorable) UnoRuntime.queryInterface(XStorable.class, doc);

			PropertyValue[] propertyvalue = new PropertyValue[2];
			propertyvalue[0] = new PropertyValue();
			propertyvalue[0].Name = "Overwrite";
			propertyvalue[0].Value = new Boolean(true);
			propertyvalue[1] = new PropertyValue();
			propertyvalue[1].Name = "FilterName";
			propertyvalue[1].Value =getDocumentFormatFilterNameText(strFormat);

			
				xstorable.storeAsURL(targetDocName, propertyvalue);
						} catch (Exception e) {
			errorMsg.append(e.toString());
			System.out.println("Error occured while saving word/pdf to a new url name  "+ e);
		}

	}
	
	private void replaceContentFromFile(XTextDocument doc, String fileContent, Object objbmk) {

		XTextRange textRange = null;
		
		try {
			try {
				
				XTextContent bookmarkContent = (XTextContent) UnoRuntime.queryInterface(XTextContent.class, objbmk);
				textRange = bookmarkContent.getAnchor();

				insertFile(doc, textRange, fileContent);
				

			} catch (Exception e) {
				
				System.out.println("ERROR occured while replacing bookmark in the process of replacing Text Fields \n "+ e);
			} 

			
		} catch (Exception e) {
			System.out.println("Exception occured while getting the tempFile "+e);
		}
	}
	
	private void insertFile(XTextDocument xtextdocument, XTextRange xTextRange, String strFileUrl) {
		try {
			PropertyValue[] insertArgs = new PropertyValue[1];
			PropertyValue args = new PropertyValue();
			args = new PropertyValue();
			args.Name = "FilterName";
			args.Value = "HTML (StarWriter)";
			insertArgs[0] = args;

			XTextCursor xTextCursor = (com.sun.star.text.XTextCursor) xtextdocument.getText().createTextCursorByRange(
					xTextRange);
			
			XDocumentInsertable xDocumentInsertable = (XDocumentInsertable) UnoRuntime.queryInterface(
					XDocumentInsertable.class, xTextCursor);
			xDocumentInsertable.insertDocumentFromURL(strFileUrl, insertArgs);

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public Object getBookMark(XTextDocument doc){
				XComponent xcomponent = (XComponent) UnoRuntime.queryInterface(XComponent.class, doc);
				XBookmarksSupplier bookmarksSupplier = (XBookmarksSupplier) UnoRuntime.queryInterface(XBookmarksSupplier.class,
						xcomponent);
				XNameAccess bookmarkNames = bookmarksSupplier.getBookmarks();
				String[] books = bookmarkNames.getElementNames();
				String strBookmarksList = "";
				if (books.length > 0) {
					for (int i = 0; i < books.length; i++) {
						strBookmarksList += "<<" + books[i] + ">>";
					}
				}
				System.out.println("Getting bookmarks : " + strBookmarksList);
				Object bmk = null;
				
				try {
					bmk=bookmarkNames.getByName("ms_file_htm");
				} catch (NoSuchElementException | WrappedTargetException e) {
					e.printStackTrace();
				}
				return bmk;
		
	}

	public void close(XTextDocument doc) {
		try {
			XCloseable xCloseable = (XCloseable) UnoRuntime.queryInterface(XCloseable.class, doc);

			if (xCloseable != null) {

				xCloseable.close(true);

				System.out.println("document closed!");
				System.exit(0);

			} else {

				doc.dispose();

				System.out.println("document closed!");
				System.exit(0);

			}

		} catch (Exception e) {
			System.out.println("Error occured while closing word document "+e);
		}

	}
     
	private String getDocumentFormatFilterNameText(String strFormat) {
		String strReturn = "MS Word 97";
		strFormat = strFormat.toUpperCase();
		if ("DOC".equals(strFormat)) {
			strReturn = "MS Word 97";
		}else if ("DOCX".equals(strFormat)){
			strReturn = "MS Word 2007 XML";
		}
		return strReturn;
	}
	private String getFileExtension(String strFileName) {

		return (strFileName.lastIndexOf(".") == -1) ? "" : strFileName.substring(strFileName.lastIndexOf(".") + 1,
				strFileName.length());
	}

	
	 public void generateDoc(StringBuffer sArgs[]) throws Exception{
		 StringBuffer errorMsg = new StringBuffer("");
		 
		 String sConnectionString = "uno:socket,host=localhost,port=8100"
					+ ";urp;StarOffice.ServiceManager";
			System.out.println("sConnectionString : " + sConnectionString);
			XTextDocument doc=open(sArgs[0].toString(), sConnectionString, errorMsg)	;
		 if (errorMsg.length() > 0) {
				System.out.println("Error: "+errorMsg);
				throw new Exception("Error in generating document");
			}
		    String tempDirectory=sArgs[3].toString();
			String outputFilename = sArgs[1].toString().substring(sArgs[1].toString().lastIndexOf('/')+1,sArgs[1].toString().length());
			File targetFile = new File(tempDirectory + outputFilename);
			StringBuffer sSaveUrl = new StringBuffer("file:///");
			sSaveUrl.append(targetFile.getCanonicalPath().replace('\\', '/'));
			Object objbmk=getBookMark(doc);
			replaceContentFromFile(doc, sArgs[2].toString(), objbmk);
			save(doc, sSaveUrl.toString(), "", errorMsg); 
			close(doc);
				
		 
	 }
	 
		 
}